@@ -21,7 +21,7 @@ weather_tsbl
 ## ----weather-tsum--------------------------------------------------------
 weather_tsbl %>%
   group_by_key() %>%
-  index_by(date = as_date(time_hour)) %>% 
+  index_by(date = ~ as_date(.)) %>% 
   summarise(
     temp_high = max(temp, na.rm = TRUE),
     temp_low = min(temp, na.rm = TRUE)
