@@ -44,7 +44,7 @@ y <- c("2016", "2016-10", "2016-12-09", "2016-12-09 10")
 tz <- "Australia/Melbourne"
 
 test_that("class: POSIXct", {
-  skip_if(is_london)
+  skip_on_cran()
   expect_error(start(pedestrian$Date_Time, 2017), "Must be")
   expect_equal(start(pedestrian$Date_Time), ymd("2015-01-01", tz = tz))
   expect_equal(
@@ -75,7 +75,7 @@ test_that("class: hms", {
 })
 
 test_that("filter_index()", {
-  skip_if(is_london)
+  skip_on_cran()
   expect_identical(
     pedestrian %>%
       filter_index(~"2015-02", "2015-08" ~ "2015-09", "2015-12" ~ .),
